﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=6
@EndOfDesignText@
#IgnoreWarnings:12
'You can set InitialFolder and initialFilterDescription before each call to a FileChooser.
'If initialFolder is not set, "user.home" of the OS will be used
'initialFolder is always setted with last opened folder
Sub Process_Globals
	Type Path (Folder As String, FileName As String)
	Private fx As JFX
	Public initialFolder As String 
	Public initialFilterDescription As String = ""
End Sub

'Thanks to @stevel05 - FileChooserUtils for this part
#Region File Chooser

'Returns uninitialized Path if a destination is not chosen
Sub SaveFile(Frm As Form, Filters As Map, AllFilters As Boolean, MergeFilters As Boolean, MergeName As String, InitialFileName As String, Title As String) As Path
	Dim p As String
	Dim toSaveFile As Path
	Dim fC As FileChooser
	
	fC.Initialize
	fC.InitialDirectory = GetInitialDirectory
	SetExtensionFilter(fC, Filters, AllFilters, MergeFilters, MergeName)
	fC.InitialFileName = InitialFileName
	fC.Title = Title
	SetSelectedExtensionFilter(fC, initialFilterDescription) 
	
	p = fC.ShowSave(Frm)
	If p = "" Then Return toSaveFile
	toSaveFile = CreatePath2(p)
	initialFolder = toSaveFile.Folder
	
	Return toSaveFile
End Sub

'Returns uninitialized Path if a destination is not chosen
'<code>	Dim filters As Map = CreateMap( 	Array As String("*.txt") : "TXT File", _
'					Array As String("*.xml") : "XML File", _
'					Array As String("*.html", "*.htm") : "Hyper Text")</code>
Sub ChooseFile(Frm As Form, Filters As Map, AllFilters As Boolean, MergeFilters As Boolean, MergeName As String, Title As String) As Path
	Dim p As String
	Dim choosenFile As Path
	Dim fc As FileChooser

	fc.Initialize
	fc.InitialDirectory = GetInitialDirectory
	SetExtensionFilter(fc, Filters, AllFilters, MergeFilters, MergeName)
	fc.Title = Title
	SetSelectedExtensionFilter(fc, initialFilterDescription) 
	
	p = fc.ShowOpen(Frm)
	If p = "" Then Return choosenFile
	choosenFile = CreatePath2(p)
	initialFolder = choosenFile.Folder 'InitialFolder is set with last opened file
	Return choosenFile
End Sub

'Returns uninitialized Path if a destination is not chosen
Sub ChooseMultipleFiles(Frm As Form, Filters As Map, AllFilters As Boolean, MergeFilters As Boolean, MergeName As String, Title As String) As List
	Dim choosenFiles As List
	Dim fc As FileChooser
	
	choosenFiles.Initialize
	
	fc.Initialize
	fc.InitialDirectory = GetInitialDirectory
	SetExtensionFilter(fc, Filters, AllFilters, MergeFilters, MergeName)
	fc.Title = Title
	SetSelectedExtensionFilter(fc, initialFilterDescription) 
	
	Dim files As List = fc.ShowOpenMultiple(Frm)
	If files.IsInitialized = False Then Return Null
	For Each fn As String In files
		choosenFiles.Add(CreatePath2(fn))
	Next
	
	Dim lastPath As Path = choosenFiles.Get(choosenFiles.Size-1)
	initialFolder = lastPath.Folder 'InitialFolder is set with last opened file
	Return choosenFiles
End Sub

'Returns uninitialized Path if a destination is not chosen
Sub ChooseFolder(Frm As Form, Title As String, InitialDir As String) As Path
	Dim p As String
	Dim choosenFolder As Path
	Dim dc As DirectoryChooser
	
	dc.Initialize
	'If InitialDir is not valid Folder, it's used initialFolder
	If InitialDir = "" Or File.IsDirectory(File.GetFileParent(InitialDir), File.GetName(InitialDir)) = False Then
		dc.InitialDirectory = GetInitialDirectory
	Else
		dc.InitialDirectory = InitialDir
	End If
	dc.Title = Title
	
	p = dc.Show(Frm)
	If p = "" Then Return choosenFolder
	choosenFolder = CreatePath2(p)
	initialFolder = choosenFolder.Folder
	Return choosenFolder
End Sub

'Filter: key = Extension, Value = Description
Private Sub SetExtensionFilter(Fc As FileChooser, Filters As Map, AllFilters As Boolean, MergeFilters As Boolean, MergeName As String)
	Dim joFileChooser As JavaObject = Fc
	Dim joExtensionFilters As List = joFileChooser.RunMethod("getExtensionFilters",Null)
	
	If AllFilters Then
		joExtensionFilters.AddAll(CreateExtensionFilters(CreateMap(Array As String("*.*") : "All Files")))
	End If

	If MergeFilters Then 
		Dim extensions As List
		extensions.Initialize
		For i = 0 To Filters.Size-1
			Dim filter() As String = Filters.GetKeyAt(i)
			For Each extension As String In filter
				extensions.Add(extension)
			Next
		Next
		joExtensionFilters.AddAll(CreateExtensionFilters(CreateMap(extensions : MergeName)))
	End If
	
	joExtensionFilters.AddAll(CreateExtensionFilters(Filters))
End Sub

'Filter: key = Extension, Value = Description
Private Sub CreateExtensionFilters(Filters As Map) As List
	Dim extensionFilters As List
	extensionFilters.Initialize
	For i = 0 To Filters.Size - 1
		Dim extensionFilter As JavaObject
		extensionFilter.InitializeNewInstance("javafx.stage.FileChooser.ExtensionFilter", Array(Filters.GetValueAt(i), (Filters.GetKeyAt(i))))
		extensionFilters.Add(extensionFilter)
	Next
	Return extensionFilters
End Sub

Private Sub GetInitialDirectory As String
	If File.IsDirectory(File.GetFileParent(initialFolder),File.GetName(initialFolder)) = False Then
		initialFolder = GetSystemProperty("user.home", "")
	End If
	Return initialFolder
End Sub

Private Sub SetSelectedExtensionFilter(FC As FileChooser, Description As String) '+17/07/18
	Dim FCJO As JavaObject = FC
	Dim EFs As List = FCJO.RunMethod("getExtensionFilters",Null)
	For Each EF As JavaObject In EFs
		If EF.RunMethod("getDescription",Null) = Description Then
			FCJO.RunMethod("setSelectedExtensionFilter",Array(EF))
			Exit
		End If
	Next
End Sub 

#End Region

#Region Path Helpers

Sub CreatePath(Folder As String, FileName As String) As Path
	Dim p As Path
	p.Initialize
	p.Folder = Folder
	p.FileName = FileName
	Return p
End Sub

Sub CreatePath2(CompletePath As String) As Path
	Dim p As Path = CreatePath(File.GetFileParent(CompletePath), File.GetName(CompletePath))
	Return p
End Sub

Sub Exist(Path As Path) As Boolean
	Return File.Exists(Path.Folder, Path.FileName)
End Sub

Sub LastModified(Path As Path) As Long
	Return File.LastModified(Path.Folder, Path.FileName)
End Sub

Sub Size(Path As Path) As Long
	Return File.Size(Path.Folder, Path.FileName)
End Sub

Sub Combine(Path As Path) As String
	Return File.Combine(Path.Folder, Path.FileName)
End Sub

Sub IsDirectory(Path As Path) As Boolean
	Return File.IsDirectory(Path.Folder, Path.FileName)
End Sub

Sub MakeDir(Path As Path)
	File.MakeDir(Path.Folder, Path.FileName)
End Sub

#End Region

#Region Files

'Check through the checksum algorithm if two files are equal.
'Algorithm = "MD5", "SHA-1", "SHA-224", "SHA-256", "SHA-384", "SHA-512" 
Sub EqualFile (Source1 As String, FileName1 As String, Source2 As String, FileName2 As String, Algorithm As String) As Boolean 
	Dim cks1 As String = GetChecksum(Source1, FileName1, Algorithm)
	Dim cks2 As String = GetChecksum(Source2, FileName2, Algorithm)
	Return cks1 = cks2
End Sub

'Algorithm can be "MD5", "SHA-1", "SHA-224", "SHA-256", "SHA-384" or "SHA-512" 
Public Sub GetChecksum(Folder As String, FileName As String, Algorithm As String) As String
	Dim buffer() As Byte = File.ReadBytes(Folder, FileName)
	Dim md As MessageDigest
	Dim Checksum() As Byte = md.GetMessageDigest(buffer, Algorithm)
	Dim bC As ByteConverter
	Return bC.HexFromBytes(Checksum)
End Sub

Sub GetFiles(FolderPath As String) As List
	If IsDirectory(CreatePath2(FolderPath)) = False Then Return Null 
	Dim files As List = File.ListFiles(FolderPath)
	Dim folders As List
	folders.Initialize
	For Each FileName As String In files
		If File.IsDirectory(FolderPath, FileName) = False Then folders.Add(FileName)
	Next
	Return folders
End Sub

'Return a new string without extension
Sub RemoveExtension(FileName As String) As String
	Dim result As String = FileName
	If result.Contains(".") Then
		Dim index As Int = result.LastIndexOf(".")
		result = result.SubString2(0,index)
	End If
	Return result
End Sub

'Return file extension
'Ex.
'FileUtils.GetExtension("test.txt") <---- Returns ".txt"
Sub GetExtension(FileName As String) As String
	Dim result As String = ""
	If FileName.Contains(".") Then
		Dim index As Int = FileName.LastIndexOf(".")
		result = FileName.SubString2(index,FileName.Length)
	End If
	Return result
End Sub

Sub OpenFile(Folder As String, FileName As String)
	OpenFile2(File.Combine(Folder, FileName))
End Sub

Sub OpenFile2(CompletePath As String)
	Dim fileJO As JavaObject
	fileJO.InitializeNewInstance("java.io.File", Array(CompletePath))
	
	Dim desktop As JavaObject
	desktop.InitializeStatic("java.awt.Desktop").RunMethodJO("getDesktop", Null).RunMethod("open", Array(fileJO))
End Sub

'Similar to File.Copy() but also folders with content can be copied
Sub Copy(DirSource As String, FileSource As String, DirTarget As String, FileTarget As String)
	If File.IsDirectory(DirSource, FileSource) Then
		Dim sourcePath As String = File.Combine(DirSource, FileSource)
		Dim targetPath As String = File.Combine(DirTarget, FileTarget)
		Dim sourceFiles As List = File.ListFiles(sourcePath)
		If sourceFiles.IsInitialized = False Then Return 'Return if folder is not accessible
		File.MakeDir(DirTarget, FileTarget)
		For Each name As String In sourceFiles
			Copy(sourcePath, name, targetPath, name) '+26/06/18
		Next
	Else
		File.Copy(DirSource, FileSource, DirTarget, FileTarget)
	End If
End Sub

'Similar to File.CopyAsync() but also folders with content can be copied
Public Sub CopyAsync (DirSource As String, FileSource As String, DirTarget As String, FileTarget As String) As ResumableSub '+26/06/18
	Dim innerSuccess As Boolean = True
	Dim sourcePath As String = File.Combine(DirSource, FileSource)
	Dim targetPath As String = File.Combine(DirTarget, FileTarget)
	If File.IsDirectory(DirSource, FileSource) Then
		Dim sourceFiles As List = File.ListFiles(sourcePath)
		If sourceFiles.IsInitialized = False Then Return False 'Return false if folder is not accessible
		File.MakeDir(DirTarget, FileTarget)
		For Each name As String In sourceFiles
			Dim obj As Object = CopyAsync(sourcePath, name, targetPath, name)
			Wait For (obj) Complete (Success As Boolean)
			innerSuccess = innerSuccess And Success
		Next
	Else
		Dim obj1 As Object = File.CopyAsync(sourcePath, name, targetPath, name)
		Wait For (obj1) Complete (Success As Boolean)
		innerSuccess = innerSuccess And Success
	End If
	Return innerSuccess
End Sub

'Similar to File.Delete() but also folders with content can be deleted
Sub Delete(Folder As String, FileName As String)
	If File.IsDirectory(Folder, FileName) Then
		Dim completePath As String = File.Combine(Folder, FileName)
		Dim files As List = File.ListFiles(completePath)
		For Each name As String In files
			If File.IsDirectory(completePath, name) Then
				Delete(completePath, name)
			End If
			File.Delete(completePath, name)
		Next
	End If
	File.Delete(Folder, FileName)
End Sub

Sub RenameTo(Folder As String, FileName As String, NewFileName As String) '+10/07/18
	Dim fileJO As JavaObject
	fileJO.InitializeNewInstance("java.io.File", Array(File.Combine(Folder, FileName)))
	Dim newFileJO As JavaObject
	newFileJO.InitializeNewInstance("java.io.File", Array(File.Combine(Folder, NewFileName)))
	fileJO.RunMethod("renameTo", Array(newFileJO))
End Sub

#End Region

#Region Folders

Sub GetFolders(FolderPath As String) As List
	If Exist(CreatePath2(FolderPath)) = False Then 
		LogError("Fille: " & FolderPath & "not exists")
		Return Null '+21/06/18
	End If
	Dim files As List = File.ListFiles(FolderPath)
	Dim folders As List
	folders.Initialize
	For Each FileName As String In files
		If File.IsDirectory(FolderPath, FileName) Then folders.Add(FileName)
	Next
	Return folders
End Sub

'Return a list of all the paths of the subfolders in rootFolder.
'The list will not contain RootFolder
Sub GetSubFolders(RootFolder As String) As List
	Dim subFolders As List
	subFolders.Initialize
	ExtractSubFolders(RootFolder, subFolders)
	Return subFolders
End Sub

'Extract subfolders from RootFolder recursively.
'Subfolder paths will be added to SubFolders
'Sub Folders will not contain RootFolder
Private Sub ExtractSubFolders(RootFolder As String, SubFolders As List)
	Dim tempList As List = File.ListFiles(RootFolder)
	For i = 0 To tempList.Size-1
		Dim watchedElement 	As String = tempList.Get(i)
		If File.IsDirectory(RootFolder, watchedElement) Then
			Dim newPath As String = File.Combine(RootFolder, watchedElement)
			SubFolders.Add(newPath)
			ExtractSubFolders(newPath,SubFolders)
		End If
	Next
End Sub

#End Region

